package io.gisbi.application.appcode.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.gisbi.application.appcode.doman.Condition;
import io.gisbi.application.appcode.doman.TableData;
import io.gisbi.chart.dao.auto.mapper.CoreChartViewMapper;
import io.gisbi.dataset.dao.auto.entity.CoreDatasetTable;
import io.gisbi.dataset.dao.auto.mapper.CoreDatasetTableMapper;
import io.gisbi.visualization.dao.auto.entity.DataVisualizationInfo;
import io.gisbi.visualization.dao.auto.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class DatabaseService {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private CoreDatasetTableMapper coreDatasetTableMapper;
    @Autowired
    private DataVisualizationInfoMapper dataVisualizationInfoMapper;
    @Autowired
    private VisualizationLinkJumpInfoMapper visualizationLinkJumpInfoMapper;
    @Autowired
    private VisualizationLinkJumpMapper visualizationLinkJumpMapper;
    @Autowired
    private CoreChartViewMapper coreChartViewMapper;
    @Autowired
    private VisualizationLinkageMapper visualizationLinkageMapper;
    @Autowired
    private VisualizationOuterParamsMapper visualizationOuterParamsMapper;
    @Autowired
    private VisualizationOuterParamsInfoMapper visualizationOuterParamsInfoMapper;

    public List<TableData> getDataById(String id) {
        List<TableData> result = new ArrayList<>();

        // 1. 获取所有要导出的表名
        List<String> tables = Arrays.asList("app_application", "app_dictionary", "app_menu", "app_module",
                "app_organization", "app_role", "app_role_menu", "app_role_users", "app_user", "area",
                "core_chart_view", "core_dataset_group", "core_dataset_table", "core_dataset_table_field",
                "core_datasource", "core_de_engine", "core_driver", "data_visualization_info",
                "visualization_background", "visualization_background_image", "visualization_link_jump",
                "visualization_link_jump_info", "visualization_link_jump_target_view_info", "visualization_linkage",
                "visualization_linkage_field", "visualization_outer_params", "visualization_outer_params_info",
                "visualization_outer_params_target_view_info", "visualization_subject");

        for (String table : tables) {

            // 2. 获取建表语句
            String createSql = getCreateTableSql(table);
            if ("app_application".equals(table)) {
                List<Condition> conditions = new ArrayList<>();
                Condition condition = new Condition();
                condition.setField("id");
                condition.setOperator("=");
                condition.setValue(id);
                conditions.add(condition);
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("area".equals(table) || "core_de_engine".equals(table) || "core_driver".equals(table) ||
                    "visualization_background".equals(table) || "visualization_background_image".equals(table)) {
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, new ArrayList<>());
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("app_dictionary".equals(table) || "app_menu".equals(table) || "app_module".equals(table) ||
                    "app_organization".equals(table) || "app_role".equals(table) || "app_role_menu".equals(table) ||
                    "app_role_users".equals(table) || "app_user".equals(table) || "core_dataset_group".equals(table) ||
                    "core_dataset_table".equals(table) || "core_datasource".equals(table) || "data_visualization_info"
                    .equals(table)) {
                List<Condition> conditions = new ArrayList<>();
                Condition condition = new Condition();
                condition.setField("app_id");
                condition.setOperator("=");
                condition.setValue(id);
                conditions.add(condition);
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("core_dataset_table_field".equals(table)) {
                List<CoreDatasetTable> coreDatasetTables =
                        coreDatasetTableMapper.selectList(new LambdaQueryWrapper<CoreDatasetTable>().eq(CoreDatasetTable::getAppId, id).select(CoreDatasetTable::getId));
                List<Condition> conditions = new ArrayList<>();
                if (!coreDatasetTables.isEmpty()) {
                    List<Long> ids =
                            coreDatasetTables.stream().map(CoreDatasetTable::getId).collect(Collectors.toList());
                    Condition condition = new Condition();
                    condition.setField("dataset_table_id");
                    condition.setOperator("in");
                    condition.setValue(ids);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("core_chart_view".equals(table)) {

                List<DataVisualizationInfo> dataVisualizationInfos =
                        dataVisualizationInfoMapper.selectList(new LambdaQueryWrapper<DataVisualizationInfo>().eq(DataVisualizationInfo::getAppId, id).select(DataVisualizationInfo::getId));
                List<Condition> conditions = new ArrayList<>();
                if (!dataVisualizationInfos.isEmpty()) {
                    List<Long> ids =
                            dataVisualizationInfos.stream().map(DataVisualizationInfo::getId).collect(Collectors.toList());
                    Condition condition = new Condition();
                    condition.setField("scene_id");
                    condition.setOperator("in");
                    condition.setValue(ids);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("visualization_link_jump".equals(table)) {
                List<DataVisualizationInfo> dataVisualizationInfos =
                        dataVisualizationInfoMapper.selectList(new LambdaQueryWrapper<DataVisualizationInfo>().eq(DataVisualizationInfo::getAppId, id).select(DataVisualizationInfo::getId));

                List<Condition> conditions = new ArrayList<>();
                if (!dataVisualizationInfos.isEmpty()) {
                    List<Long> ids =
                            dataVisualizationInfos.stream().map(DataVisualizationInfo::getId).collect(Collectors.toList());
                    Condition condition = new Condition();
                    condition.setField("source_dv_id");
                    condition.setOperator("in");
                    condition.setValue(ids);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));

            } else if ("visualization_link_jump_info".equals(table)) {
                List<Long> ids = visualizationLinkJumpMapper.selectLinkJumpIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!ids.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("link_jump_id");
                    condition.setOperator("in");
                    condition.setValue(ids);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else if ("visualization_link_jump_target_view_info".equals(table)) {
                List<Long> dvInfoIds = visualizationLinkJumpInfoMapper.selectInfoIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!dvInfoIds.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("link_jump_info_id");
                    condition.setOperator("in");
                    condition.setValue(dvInfoIds);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            }else if ("visualization_linkage".equals(table)) {
                List<Long> viewIds = coreChartViewMapper.selectChatViewIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!viewIds.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("source_view_id");
                    condition.setOperator("in");
                    condition.setValue(viewIds);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            }else if ("visualization_linkage_field".equals(table)) {
                List<Long> linkageIds = visualizationLinkageMapper.selectLinkageIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!linkageIds.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("linkage_id");
                    condition.setOperator("in");
                    condition.setValue(linkageIds);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            }else if ("visualization_outer_params".equals(table)) {
                List<DataVisualizationInfo> dataVisualizationInfos =
                        dataVisualizationInfoMapper.selectList(new LambdaQueryWrapper<DataVisualizationInfo>().eq(DataVisualizationInfo::getAppId, id).select(DataVisualizationInfo::getId));
                List<Condition> conditions = new ArrayList<>();
                if (!dataVisualizationInfos.isEmpty()) {
                    List<Long> ids =
                            dataVisualizationInfos.stream().map(DataVisualizationInfo::getId).collect(Collectors.toList());
                    Condition condition = new Condition();
                    condition.setField("visualization_id");
                    condition.setOperator("in");
                    condition.setValue(ids);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            }else if ("visualization_outer_params_info".equals(table)) {
                List<Long> outerParamIds = visualizationOuterParamsMapper.selectOuterParamIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!outerParamIds.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("params_id");
                    condition.setOperator("in");
                    condition.setValue(outerParamIds);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            }else if ("visualization_outer_params_target_view_info".equals(table)) {
                List<Long> paramsInfoIds = visualizationOuterParamsInfoMapper.selectParamsInfoIds(id);
                List<Condition> conditions = new ArrayList<>();
                if (!paramsInfoIds.isEmpty()) {
                    Condition condition = new Condition();
                    condition.setField("params_info_id");
                    condition.setOperator("in");
                    condition.setValue(paramsInfoIds);
                    conditions.add(condition);
                }
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, conditions);
                result.add(new TableData(table, createSql, insertSqls));
            } else {
                // 3. 获取数据插入语句
                List<String> insertSqls = getInsertSqls(table, new ArrayList<>());
                result.add(new TableData(table, createSql, insertSqls));
            }
        }
        return result;
    }

    private String getCreateTableSql(String table) {
        Map<String, Object> row = jdbcTemplate.queryForMap("SHOW CREATE TABLE " + table);
        String createTableSql = (String) row.get("Create Table");

        // 在建表语句前加上 DROP TABLE IF EXISTS
        return "DROP TABLE IF EXISTS `" + table + "`;\n" + createTableSql + ";\n";
    }

    private List<String> getInsertSqls(String table, List<Condition> conditions) {
        StringBuilder whereClause = new StringBuilder();
        List<Object> params = new ArrayList<>();
        if (conditions != null && !conditions.isEmpty()) {
            whereClause.append(" WHERE ");
            for (int i = 0; i < conditions.size(); i++) {
                Condition condition = conditions.get(i);
                if (i > 0) {
                    whereClause.append(" AND ");
                }
                String operator = condition.getOperator().trim().toLowerCase();
                if ("in".equalsIgnoreCase(operator)) {
                    // 处理 IN 查询，动态生成占位符
                    @SuppressWarnings("unchecked")
                    List<Object> inValues = (List<Object>) condition.getValue();
                    String placeholders = inValues.stream()
                            .map(v -> "?")
                            .collect(Collectors.joining(", "));
                    whereClause.append(condition.getField()).append(" IN (").append(placeholders).append(")");
                    params.addAll(inValues);
                } else {
                    // 处理其他操作符
                    whereClause.append(condition.getField())
                            .append(" ")
                            .append(condition.getOperator())
                            .append(" ?");
                    params.add(condition.getValue());
                }
            }
        }

        String sql = "SELECT * FROM " + table + whereClause;

        List<Map<String, Object>> rows = jdbcTemplate.queryForList(sql, params.toArray());

        List<String> inserts = new ArrayList<>();

        // 添加 INSERT 语句
        for (Map<String, Object> row : rows) {
            StringBuilder columns = new StringBuilder();
            StringBuilder values = new StringBuilder();

            row.forEach((key, value) -> {
                columns.append("`").append(key).append("`");
                switch (value) {
                    case String s ->
                        // 对字符串进行转义
                            values.append(escapeSql(s));
                    case null -> values.append("NULL");
                    case Number ignored -> values.append(value); // 数字不需要加引号
                    default ->
                        // 其他类型统一转为字符串处理
                            values.append(escapeSql(value.toString()));
                }

                if (!key.equals(row.keySet().toArray()[row.size() - 1])) {
                    values.append(", ");
                    columns.append(", ");
                }
            });

            inserts.add("INSERT INTO `" + table + "` (" + columns + ") VALUES (" + values + ");");
        }

        inserts.add("\n"); // 每个表之间空一行
        return inserts;
    }

    public String escapeSql(String value) {
        if (value == null) {
            return "NULL";
        }

        int len = value.length();
        StringBuilder out = new StringBuilder(len + 16);

        for (int i = 0; i < len; i++) {
            char c = value.charAt(i);
            switch (c) {
                case '\'':
                    out.append("''");
                    break;
                case '\\':
                    out.append("\\\\");
                    break;
                case '\0':
                    out.append("\\0");
                    break;
                case '\n':
                    out.append("\\n");
                    break;
                case '\r':
                    out.append("\\r");
                    break;
                case '\t':
                    out.append("\\t");
                    break;
                case '\b':
                    out.append("\\b");
                    break;
                case '\u001A': // Ctrl+Z
                    out.append("\\Z");
                    break;
                case '"':
                    out.append("\\\"");
                    break;
                case '\u001B':
                    out.append("\\e");
                    break;
                default:
                    out.append(c);
            }
        }

        return "'" + out.toString() + "'";
    }

}
